import torch
from torch import nn
import numpy as np
from structures import *
from models import *
import time

# loads dataset after preprocessing
train_data = PreprocessedDataSet(['/workspace/students/kreuzer/train0','/workspace/students/kreuzer/train1','/workspace/students/kreuzer/train2','/workspace/students/kreuzer/train3','/workspace/students/kreuzer/train4','/workspace/students/kreuzer/train5','/workspace/students/kreuzer/train6','/workspace/students/kreuzer/train7','/workspace/students/kreuzer/train8','/workspace/students/kreuzer/train9'])
val_data = PreprocessedDataSet(['/workspace/students/kreuzer/val'])
test_data = PreprocessedDataSet(['/workspace/students/kreuzer/test'])

# hyperparameters
epochs=20 # 20
batch_size=20 # 20
learning_rate=0.001


train_dataloader = torch.utils.data.DataLoader(train_data, batch_size=batch_size, shuffle=True, collate_fn = lambda x:x) # collate_fn 
#test_dataloader = torch.utils.data.DataLoader(test_data, batch_size=batch_size, shuffle=True, collate_fn = lambda x: x)


m = ActorOnlySummarisationModel() # tip error fix!

since = time.time()
val_rouge_history = []

best_rouge = 0.0
best_model_wts = copy.deepcopy(m.state_dict()) 



for epoch in range(epochs):

    print()
    print('Epoch {}/{}'.format(epoch+1, epochs))
    print('-' * 10)

    # train phase
    epoch_loss, epoch_rouge = m.training_epoch(train_dataloader) # train_dataloader
    print('Train Loss: {:.4f} Rouge Score: {:.4f}'.format(epoch_loss, epoch_rouge))
    
    # validation phase 
    epoch_rouge = m.validation(val_data) #
    val_rouge_history.append(epoch_rouge)    

    print('Validation Rouge Score: {:.4f}'.format(epoch_rouge))

    # epoch completed, deep copy the best model sofar
    if epoch_rouge > best_rouge:
        best_rouge = epoch_rouge
        best_model_wts = copy.deepcopy(m.state_dict())
        
        # (over)write best model in file, and for Critic: save model_actor_only   
        torch.save(m.state_dict(), 'model_actor_only_wts.pth')

        # load best model and compute sent_vecs anew
        best_m = ActorOnlySummarisationModel()
        best_m.load_state_dict(best_model_wts)
        train_data.compute_sent_vecs(best_m, 'workspace/students/kreuzer/new_train')  #
        val_data.compute_sent_vecs(best_m, 'workspace/students/kreuzer/new_val')
        test_data.compute_sent_vecs(best_m, 'workspace/students/kreuzer/new_test')

    time_elapsed = time.time() - since
    print('Training + validation running already {:.0f}m {:.0f}s'.format(time_elapsed // 60, time_elapsed % 60))
    print('Best val rouge so far: {:4f}'.format(best_rouge))
    
    # save model weights after each validation
    torch.save(m.state_dict(), f'model_actor_only_wts{epoch}.pth')
    
   # (over)write val_rouge_history in file
    torch.save(val_rouge_history, 'model_actor_only_val_rouge_hist.pt') 


# after training completed
time_elapsed = time.time() - since
print('Training + validation complete in {:.0f}m {:.0f}s'.format(time_elapsed // 60, time_elapsed % 60))
print('Best val rouge: {:4f}'.format(best_rouge))




# testing
since = time.time()

epoch_rouge_1, epoch_rouge_2, epoch_rouge_l = m.test(test_data)
print('Test rouge_1: {:.4f} rouge_2: {:.4f} rouge_l: {:.4f} mean: {:.4f}'.format(epoch_rouge_1, epoch_rouge_2, epoch_rouge_l, (epoch_rouge_1+epoch_rouge_2+epoch_rouge_l)/3.0))

# after testing completed
time_elapsed = time.time() - since
print('Testing complete in {:.0f}m {:.0f}s'.format(time_elapsed // 60, time_elapsed % 60))
