import torch
from torch import nn
import numpy as np
from structures import *
from models import *
import time

# loads dataset cnn_dailymail
train_dataset = PreprocessedDataSet(['/workspace/students/kreuzer/new_train'])
validation_dataset = PreprocessedDataSet(['/workspace/students/kreuzer/new_val'])
test_dataset = PreprocessedDataSet(['/workspace/students/kreuzer/new_test'])



# hyperparameters
epochs=10 # <= actor trained epochs 
batch_size=20
learning_rate=0.001

train_dataloader = torch.utils.data.DataLoader(train_dataset, batch_size=batch_size, shuffle=True, collate_fn = lambda x: x)

# load state dict from training
model_actor_only = ActorOnlySummarisationModel()
model_actor_only.load_state_dict(torch.load('model_actor_only_wts.pth')) #blow up here?
model_actor_only.eval()

m = Critic(model_actor_only)


since = time.time()
val_loss_history = []

best_model_wts = copy.deepcopy(m.state_dict())

for e in range(epochs):

    print()
    print('Epoch {}/{}'.format(e+1, epochs))
    print('-' * 10)

    # train phase
    epoch_loss = m.training_epoch(train_dataloader)  # collate_fn
    print('Train Loss: {:.4f}'.format(epoch_loss))
    
    # validation phase 
    val_epoch_loss = m.test(validation_dataset)
    val_loss_history.append(val_epoch_loss)    
    print('Validation Loss: {:.4f}'.format(val_epoch_loss))

    # epoch completed, deep copy the best model sofar
    if val_epoch_loss < best_loss:
        best_loss = val_epoch_loss
        best_model_wts = copy.deepcopy(m.state_dict())
        # (over)write best model in file
        torch.save(m.state_dict(), 'model_critic_wts.pth')
    
    time_elapsed = time.time() - since
    print('Training + validation running already {:.0f}m {:.0f}s'.format(time_elapsed // 60, time_elapsed % 60))
    print('Best val result so far: {:4f}'.format(best_loss))

    # save model weights after each validation
    torch.save(m.state_dict(), f'model_critic_wts{epoch}.pth')
    
    # (over)write val_rouge_history in file
    torch.save(val_loss_history, 'model_critic_val_loss_hist.pt') 


# after training completed
time_elapsed = time.time() - since
print('Training complete in {:.0f}m {:.0f}s'.format(time_elapsed // 60, time_elapsed % 60))
print('Best val loss: {:4f}'.format(best_loss))
# write val_rouge_history in file

# load best model weights
m.load_state_dict(best_model_wts)


# testing
since = time.time()
test_loss = m.test(test_dataset)
print('Test rouge score difference: {:.4f}'.format(test_loss))

# after testing completed
time_elapsed = time.time() - since
print('Testing complete in {:.0f}m {:.0f}s'.format(time_elapsed // 60, time_elapsed % 60))
